;no dim9
;no m(maj7)

#include "Ti86.inc"

;MEM LOCATIONS (vars, if you want ;-)  )

Input_Str_Save_SP	equ	$ca00
Base_Note			equ	$ca02
Status				equ	$ca03
Scale_Type			equ	$ca04
Note_Table_Addr		equ	$ca05
Chord_Buffer		equ	$ca07
Str_Buffer			equ	$ca17

;bits in "status"
bit_sharp_flat		equ	0	;(sharp if 0, flat if 1)
bit_maj				equ	1	;(off if 0, on if 1)
bit_dim				equ	2
bit_m				equ	3
bit_56				equ	4
bit_showscale		equ 5	;(on if 0, off if 1)
bit_showchord		equ 6

;values for "scale_type"
typ_M				equ	0
typ_m				equ	1
typ_dim				equ	2
typ_pent			equ	3
typ_mpent			equ	4

;CURSOR POSITIONS

Title_1_Pos			equ	$0000
Title_2_Pos			equ	$0716
Input_Str_Pos		equ	$1400
Menu_Pos			equ	$3900
Error_Pos			equ	$2200
Output_Pos			equ	$2200
Output_Pos_2		equ	$2900
Text_Win_Top		equ	2
Text_Win_Bot		equ	6

;b for m (3), dim, maj (7, 9, 11, 13)
;# for aug

;0=ab
;1=a
;2=bb
;3=b
;4=c
;5=db
;6=d
;7=eb
;8=e
;9=f
;10=gb
;11=g

;0=M
;1=m
;3=
;4=pent
;5=mpent

.org _asm_exec_ram

	nop
	jp Prog_Start

.dw 0,Shell_Title

Title_Text_1: .db "   "
Shell_Title: 	.db "Chord/Scale Calc. by"
				.db " Donald Straney   ",0
Title_Text_2: .db "straney",$40,"umbi.umd.edu",0

Prog_Start:
;normal init...
	call _runindicoff	;This part should be obvious...
	call _flushallmenus
	call _clrLCD
;draw title bars, etc.
	set textEraseBelow,(iy+textflags)
	ld hl,Title_1_Pos
	ld (_penCol),hl
	set textInverse,(iy+textflags)
	ld hl,Title_Text_1
	call _vputs
	res textInverse,(iy+textflags)
	ld hl,Title_2_Pos
	ld (_penCol),hl
	ld hl,Title_Text_2
	call _vputs
	ld hl,Menu_Pos
	ld (_penCol),hl
	ld hl,Menu_Str
	call _vputs
;get string...
Get_Str:
	xor a					;Turn off all options for chord/scale
	ld (Status),a
	ld a,Text_Win_Top		;Clear lines where text will be entered and displayed
	ld (_winTop),a
	ld a,Text_Win_Bot
	ld (_winBtm),a
	call _clrWindow
	ld hl,Input_Str_Pos
	ld (_penCol),hl
	ld de,Str_Buffer
	call Input_Str			;Get string
	ld ix,Status
;start parsing!
	ld hl,Str_Buffer
	ld a,(hl)
	or a					;Is there any text?
	jp z,Error				;If not, show error
;first get base note
	ld b,a					;Otherwise, base note should be first
	and $f0
	cp $e0
	ld a,b
	jp nz,Error				;If one of the base note tokens isn't first, show error
	and $0f					;Otherwise, if everything's fine, get number of base note from token
	ld (Base_Note),a		;and store that
	inc hl
	ld a,(hl)
Test_Is_Flat:
	cp $f6					;Is is a flat?
	jr nz,Test_Is_Sharp
	ld a,(Base_Note)		;If so, correct base note
	dec a
	ld (Base_Note),a
	inc hl					;Point to next token
	set bit_sharp_flat,(ix+0)	;Select flats
	jr Continue_1
Test_Is_Sharp:
	cp $f7					;Is this a sharp?
	jr nz,Is_Natural
	ld a,(Base_Note)		;If so, correct base note
	inc a
	ld (Base_Note),a
	inc hl					;Point to next token
	jr Continue_1			;(sharps are selected by default, we don't have to change anything)
Is_Natural:
	ld a,(Base_Note)
	cp 9					;Is the base note an f?
	jr nz,Continue_1
	set bit_sharp_flat,(ix+0)	;If so, set for flats
Continue_1:
;now check for dim, aug, m, maj, 5, 6, generate basic structure of chord from that
	ld de,Chord_Buffer			;Set pointer to start of chord buffer
	ex de,hl					;Swap chord buffer and string buffer pointers
	ld a,(de)					;Check next token
	or a						;Is it the end of the string?
	jr nz,Continue_2
	call Modify_M				;If so, generate chord and display it!!!
	jr Calc_It
Continue_2:
	cp $f5						;Is it augmented?
	jp z,Modify_aug
	cp $f4						;Is it minor?
	jp z,Modify_m
	cp $f3						;Is it a maj7 or 9?
	jp z,Modify_maj
	cp $f2						;Is it diminished?
	jp z,Modify_dim
	cp $05						;Is it a power chord?
	jp z,Modify_5
	cp $06						;Is it a 6th?
	jp z,Modify_6
	cp $f8						;Is it a pentatonic?
	jp z,Modify_pent
	call Modify_M				;If none of these, it's major
Continue_3:
;check for m. pent, m6th, 7th, 9th, 11th, or 13th chords
	inc de						;Look at next token
	ld a,(de)
	or a						;Is this the end of the string?
	jr nz,Continue_4			;(check for incomplete notation)
	bit bit_maj,(ix+0)			;If so, was the previous token a "maj"?
	jp nz,Error					;If it was, show error
	jr Calc_It					;If the notation wasn't incomplete, calculate the chord
Continue_4:
	cp $f8						;Is it a minor pentatonic?
	jp z,Modify_mpent
	cp $06						;Is it a minor 6th?
	jp z,Modify_m6
	cp $07						;Is it a 7th chord?
	jp z,Modify_7
	cp $09						;Is it a 9th chord?
	jp z,Modify_9
	cp $01						;Is it a (11/13)?
	jp z,Modify_1
	dec de
	bit bit_maj,(ix+0)
	jp nz,Error					;If there was a "maj" but no 7 or 9, show error
Continue_5:
;check for additional "add9" and "sus"s
	inc de						;Look at next token
	ld a,(de)
	or a						;Is it the end of the string?
	jr z,Calc_It				;If so, calculate chord
	cp $f0						;Is it a sus chord?
	jp z,Modify_sus
	cp $f1						;Is it an add9 chord?
	jp z,Modify_add9
	jp Error					;If it's neither, show error
Calc_It:
	ld (hl),0					;Zero-terminate chord note buffer
	ld hl,Chord_Buffer			;Start pointer back at beginning of chord buffer
	ld de,Chord_Text_Table_Sharp	;Select appropriate table (flats vs. sharps)
	bit bit_sharp_flat,(ix+0)
	jr z,Calc_Chord_Set_Table
	ld de,Chord_Text_Table_Flat
Calc_Chord_Set_Table:
	ld (Note_Table_Addr),de
	bit bit_showchord,(ix+0)	;Is a chord supposed to be shown?
	jr nz,Calc_Scale			;If not, skip that
	ld de,Output_Pos			;Otherwise, position cursor
	push ix
	call Calc_Scale_Chord		;and calculate, draw chord
	pop ix
Calc_Scale:
	ld a,(Chord_Buffer+1)		;Set for flats if there is an accidental (scales look much better that way)
	cp $f7
	jr nz,Calc_Scale_0
	ld de,Chord_Text_Table_Flat
	ld (Note_Table_Addr),de
Calc_Scale_0:
	bit bit_showscale,(ix+0)	;Can we display the scale?
	jr nz,End_Calc_Scale		;If not, skip that step
	ld a,(Scale_Type)			;What type of scale?
	or a						;Is it major?
	jr nz,Calc_Scale_1
	ld hl,M_scale
	jr Really_Calc_Scale
Calc_Scale_1:
	dec a						;Is it minor?
	jr nz,Calc_Scale_2
	ld hl,m_scale
	jr Really_Calc_Scale
Calc_Scale_2:
	dec a						;Is it diminished?
	jr nz,Calc_Scale_3
	ld hl,dim_scale
	jr Really_Calc_Scale
Calc_Scale_3:
	dec a						;Is is major pentatonic?
	jr nz,Calc_Scale_4
	ld hl,pent_scale
	jr Really_Calc_Scale
Calc_Scale_4:
	dec a						;Is it minor pentatonic?
	jp nz,Error					;If it's none of these, something has seriously screwed up
	ld hl,mpent_scale
Really_Calc_Scale:
	ld de,Output_Pos_2			;Position cursor
	call Calc_Scale_Chord		;Calculate and draw scale
End_Calc_Scale:
	call _getkey
	cp kExit
	jp z,_clrLCD
	jp Get_Str

;SCALE/CHORD DATA LOAD ROUTINES

Modify_aug:
	set bit_showscale,(ix+0)
	ld (hl),aug_1				;Load chord buffer with basic chord structure
	inc hl
	ld (hl),aug_2
	inc hl
	jp Continue_3

Modify_m:
	set bit_m,(ix+0)
	ld a,typ_m
	ld (Scale_Type),a
	ld (hl),m_1
	inc hl
	ld (hl),m_2
	inc hl
	jp Continue_3

Modify_maj:
	set bit_maj,(ix+0)
	set bit_showscale,(ix+0)
	call Modify_M
	inc de
	jp Continue_3

Modify_dim:
	set bit_dim,(ix+0)
	ld a,typ_dim
	ld (Scale_Type),a
	ld (hl),dim_1
	inc hl
	ld (hl),dim_2
	inc hl
	jp Continue_3

Modify_5:
	set bit_showscale,(ix+0)
	set bit_56,(ix+0)
	ld (hl),c5_1
	inc hl
	jp Continue_5

Modify_6:
	set bit_showscale,(ix+0)
	set bit_56,(ix+0)
	ld (hl),c6_1
	inc hl
	ld (hl),c6_2
	inc hl
	ld (hl),c6_3
	inc hl
	jp Continue_5

Modify_sus:
	bit bit_dim,(ix+0)
	jp nz,Error
	bit bit_56,(ix+0)
	jp nz,Error
	set bit_showscale,(ix+0)
	inc de
	ld a,(de)
	cp $02
	jr z,Modify_sus2
	cp $04
	jr z,Modify_sus4
	or a
	jr z,Modify_sus4
	jp Error
Modify_sus4:
	ld a,sus4_1
	ld (Chord_Buffer),a
	jp Continue_5
Modify_sus2:
	ld a,sus2_1
	ld (Chord_Buffer),a
	jp Continue_5

Modify_M:
	xor a
	ld (Scale_Type),a
	ld (hl),M_1
	inc hl
	ld (hl),M_2
	inc hl
	dec de
	ret

Modify_7:
	set bit_showscale,(ix+0)
	bit bit_maj,(ix+0)
	jr nz,Modify_7_maj
	bit bit_dim,(ix+0)
	jr nz,Modify_7_dim
	ld (hl),c7_1
	inc hl
	jp Continue_5
Modify_7_maj:
	ld (hl),c7maj_1
	inc hl
	jp Continue_5
Modify_7_dim:
	ld (hl),c7dim_1
	inc hl
	jp Continue_5

Modify_9:
	set bit_showscale,(ix+0)
	bit bit_maj,(ix+0)
	jr nz,Modify_9_maj
	bit bit_dim,(ix+0)
	jr nz,Error
	ld (hl),c9_1
	inc hl
	ld (hl),c9_2
	inc hl
	jp Continue_5
Modify_9_maj:
	ld (hl),c9maj_1
	inc hl
	ld (hl),c9maj_2
	inc hl
	jp Continue_5

Modify_1:
	set bit_showscale,(ix+0)
	bit bit_maj,(ix+0)
	jr nz,Error
	bit bit_dim,(ix+0)
	jr nz,Error
	inc de
	ld a,(de)
	cp $01
	jr z,Modify_11
	cp $03
	jr z,Modify_13
	jr Error

Modify_11:
	ld (hl),c11_1
	inc hl
	ld (hl),c11_2
	inc hl
	ld (hl),c11_3
	inc hl
	jp Continue_5

Modify_13:
	ld (hl),c13_1
	inc hl
	ld (hl),c13_2
	inc hl
	ld (hl),c13_3
	inc hl
	ld (hl),c13_4
	inc hl
	jp Continue_5

Modify_pent:
	set bit_showchord,(ix+0)
	ld a,typ_pent
	ld (Scale_Type),a
	jp Calc_It

Modify_mpent:
	set bit_showchord,(ix+0)
	bit bit_m,(ix+0)
	jr z,Error
	ld a,typ_mpent
	ld (Scale_Type),a
	jp Calc_It

Modify_add9:
	set bit_showscale,(ix+0)
	ld (hl),c9_2
	inc hl
	jp Continue_5

Modify_m6:
	set bit_showscale,(ix+0)
	ld (hl),c6_3
	inc hl
	jp Continue_5

;OTHER ROUTINES

Error:
	ld hl,Error_Pos
	ld (_penCol),hl
	ld hl,Error_Str
	call _vputs
	call _getkey
	jp Get_Str

Disp_Chord_Name:				;Display name of note in a
	ld b,a
	add a,b
	add a,b
	ld l,a
	ld h,0
	ld de,(Note_Table_Addr)
	add hl,de
	jp _vputs

Calc_Scale_Chord:				;Calculate, display chord/scale from 0-terminated list at (hl), cursor position in de
	ld (_penCol),de
	ld a,(Base_Note)
	ld (Patch_1+1),a
Calc_Chord_Loop:
	push hl
	call Disp_Chord_Name
	ld a,Lcomma
	call _vputmap
	pop hl
Patch_1:
	ld a,0
	add a,(hl)
Calc_Chord_Dec_Loop:
	cp 12
	jr c,Calc_Chord_Skip_Dec
	sub 12
	jr Calc_Chord_Dec_Loop
Calc_Chord_Skip_Dec:
	inc hl
	ld b,(hl)
	inc b
	djnz Calc_Chord_Loop
	jr Disp_Chord_Name

Input_Str:
	ld (Input_Str_Save_SP),sp
	ex de,hl
Input_Str_Loop:
	push hl
	ld a,(_penCol)
	push af
	ld hl,Cursor_Str
	call _vputs
	pop af
	ld (_penCol),a
	call _getkey
	ld b,a
	pop hl
	cp kExit
	jr z,Input_Str_Exit
	cp kEnter
	jr z,Input_Str_Enter
	cp kDel
	jr z,Input_Str_Del
	ld de,Input_Str_Key_Table
Input_Str_Find_Loop:
	ld a,(de)
	cp $d6
	jr z,Input_Str_Loop
	cp b
	jr z,Input_Str_Found
Input_Str_Find_Loop_2:
	inc de
	ld a,(de)
	or a
	jr nz,Input_Str_Find_Loop_2
	inc de
	jr Input_Str_Find_Loop
Input_Str_Found:
	inc de
	ld a,(de)
	ld (hl),a
	inc hl
	inc de
	ld a,(_penCol)
	push af
	push hl
	ex de,hl
	call _vputs
	pop hl
	jr Input_Str_Loop
Input_Str_Exit:
	ld hl,(Input_Str_Save_SP)
	ld sp,hl
	pop hl
	jp _clrLCD
Input_Str_Enter:
	ld (hl),0
	ld hl,(Input_Str_Save_SP)
	ld sp,hl
	ret
Input_Str_Del:
	pop af
	ld (_penCol),a
	dec hl
	jr Input_Str_Loop

;DATA!!!

Input_Str_Key_Table:
.db kF1,$f0,"sus",0
.db kF2,$f1,"add9",0
.db kF3,$f2,Ldegree,0
.db kF4,$f3,"maj",0
.db kF5,$f4,"m",0
.db kAdd,$f5,"+",0
.db kGrMenu,$f6,"b",0
.db kTable,$f7,"#",0
.db kPrgm,$f8,"pent",0
.db kLog,$e1,"A",0
.db kSin,$e3,"B",0
.db kCos,$e4,"C",0
.db kTan,$e6,"D",0
.db kExpon,$e8,"E",0
.db kLn,$e9,"F",0
.db kEE,$eb,"G",0
.db k1,$01,"1",0
.db k2,$02,"2",0
.db k3,$03,"3",0
.db k4,$04,"4",0
.db k5,$05,"5",0
.db k6,$06,"6",0
.db k7,$07,"7",0
;.db k8,$08,"8",0
.db k9,$09,"9",0
;.db k0,$0a,"0",0
.db $d6

Menu_Str:	.db "   sus                 "
			.db "add9                  ",Ldegree
			.db "                  maj  "
			.db "                m",0
Cursor_Str:	.db Lunderscore,"                ",0
Error_Str: .db "Input error!",0

Chord_Text_Table_Flat:
.db "Ab",0
.db "A ",0
.db "Bb",0
.db "B ",0
.db "C ",0
.db "Db",0
.db "D ",0
.db "Eb",0
.db "E ",0
.db "F ",0
.db "Gb",0
.db "G ",0

Chord_Text_Table_Sharp:
.db "G#",0
.db "A ",0
.db "A#",0
.db "B ",0
.db "C ",0
.db "C#",0
.db "D ",0
.db "D#",0
.db "E ",0
.db "F ",0
.db "F#",0
.db "G ",0

M_scale: .db 2,4,5,7,9,11,0
m_scale: .db 2,3,5,7,8,10,0	;whwwhwh
dim_scale: .db 2,3,5,6,8,9,0	;whwhwhwh
pent_scale: .db 2,4,7,9,0
mpent_scale: .db 3,5,7,10,0

M_1	equ 4
M_2	equ	7	;major

m_1	equ	3
m_2	equ	7	;minor

aug_1	equ	4
aug_2	equ	8	;augmented

dim_1	equ 3
dim_2	equ	6	;diminished

sus4_1	equ 5	;sus4?

sus2_1	equ	2	;sus2?

c5_1	equ	7	;5

c6_1	equ	4
c6_2	equ	7
c6_3	equ	9	;6

c7_1	equ	10	;7
c7maj_1	equ	11
c7dim_1	equ	9

c9_1	equ	10
c9_2	equ	14	;9
c9maj_1	equ	11
c9maj_2	equ	15

c11_1	equ	10	;11
c11_2	equ	14
c11_3	equ	5

c13_1	equ	10	;13
c13_2	equ	14
c13_3	equ	5
c13_4	equ	9

.end
